package org.springframework.boot.loader.jar;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.net.JarURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLStreamHandler;
import java.security.Permission;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;
import org.springframework.boot.loader.jar.AbstractJarFile;
import org.springframework.boot.loader.jar.JarEntry;
import org.springframework.boot.loader.jar.JarFile;
import org.springframework.boot.loader.jar.StringSequence;

final class JarURLConnection extends JarURLConnection {
  private static ThreadLocal<Boolean> useFastExceptions = new ThreadLocal<>();
  
  private static final FileNotFoundException FILE_NOT_FOUND_EXCEPTION = new FileNotFoundException("Jar file or entry not found");
  
  private static final IllegalStateException NOT_FOUND_CONNECTION_EXCEPTION = new IllegalStateException(FILE_NOT_FOUND_EXCEPTION);
  
  private static final String SEPARATOR = "!/";
  
  private static final URL EMPTY_JAR_URL;
  
  static {
    try {
      EMPTY_JAR_URL = new URL("jar:", null, 0, "file:!/", (URLStreamHandler)new Object());
    } catch (MalformedURLException ex) {
      throw new IllegalStateException(ex);
    } 
  }
  
  private static final JarEntryName EMPTY_JAR_ENTRY_NAME = new JarEntryName(new StringSequence(""));
  
  private static final org.springframework.boot.loader.jar.JarURLConnection NOT_FOUND_CONNECTION = notFound();
  
  private final AbstractJarFile jarFile;
  
  private Permission permission;
  
  private URL jarFileUrl;
  
  private final JarEntryName jarEntryName;
  
  private JarEntry jarEntry;
  
  private JarURLConnection(URL url, AbstractJarFile jarFile, JarEntryName jarEntryName) throws IOException {
    super(EMPTY_JAR_URL);
    this.url = url;
    this.jarFile = jarFile;
    this.jarEntryName = jarEntryName;
  }
  
  public void connect() throws IOException {
    if (this.jarFile == null)
      throw FILE_NOT_FOUND_EXCEPTION; 
    if (!this.jarEntryName.isEmpty() && this.jarEntry == null) {
      this.jarEntry = this.jarFile.getJarEntry(getEntryName());
      if (this.jarEntry == null)
        throwFileNotFound(this.jarEntryName, this.jarFile); 
    } 
    this.connected = true;
  }
  
  public JarFile getJarFile() throws IOException {
    connect();
    return (JarFile)this.jarFile;
  }
  
  public URL getJarFileURL() {
    if (this.jarFile == null)
      throw NOT_FOUND_CONNECTION_EXCEPTION; 
    if (this.jarFileUrl == null)
      this.jarFileUrl = buildJarFileUrl(); 
    return this.jarFileUrl;
  }
  
  private URL buildJarFileUrl() {
    try {
      String spec = this.jarFile.getUrl().getFile();
      if (spec.endsWith("!/"))
        spec = spec.substring(0, spec.length() - "!/".length()); 
      if (!spec.contains("!/"))
        return new URL(spec); 
      return new URL("jar:" + spec);
    } catch (MalformedURLException ex) {
      throw new IllegalStateException(ex);
    } 
  }
  
  public JarEntry getJarEntry() throws IOException {
    if (this.jarEntryName == null || this.jarEntryName.isEmpty())
      return null; 
    connect();
    return this.jarEntry;
  }
  
  public String getEntryName() {
    if (this.jarFile == null)
      throw NOT_FOUND_CONNECTION_EXCEPTION; 
    return this.jarEntryName.toString();
  }
  
  public InputStream getInputStream() throws IOException {
    if (this.jarFile == null)
      throw FILE_NOT_FOUND_EXCEPTION; 
    if (this.jarEntryName.isEmpty() && this.jarFile.getType() == AbstractJarFile.JarFileType.DIRECT)
      throw new IOException("no entry name specified"); 
    connect();
    InputStream inputStream = this.jarEntryName.isEmpty() ? this.jarFile.getInputStream() : this.jarFile.getInputStream(this.jarEntry);
    if (inputStream == null)
      throwFileNotFound(this.jarEntryName, this.jarFile); 
    return inputStream;
  }
  
  private void throwFileNotFound(Object entry, AbstractJarFile jarFile) throws FileNotFoundException {
    if (Boolean.TRUE.equals(useFastExceptions.get()))
      throw FILE_NOT_FOUND_EXCEPTION; 
    throw new FileNotFoundException("JAR entry " + entry + " not found in " + jarFile.getName());
  }
  
  public int getContentLength() {
    long length = getContentLengthLong();
    if (length > 2147483647L)
      return -1; 
    return (int)length;
  }
  
  public long getContentLengthLong() {
    if (this.jarFile == null)
      return -1L; 
    try {
      if (this.jarEntryName.isEmpty())
        return this.jarFile.size(); 
      JarEntry entry = getJarEntry();
      return (entry != null) ? (int)entry.getSize() : -1L;
    } catch (IOException ex) {
      return -1L;
    } 
  }
  
  public Object getContent() throws IOException {
    connect();
    return this.jarEntryName.isEmpty() ? this.jarFile : super.getContent();
  }
  
  public String getContentType() {
    return (this.jarEntryName != null) ? this.jarEntryName.getContentType() : null;
  }
  
  public Permission getPermission() throws IOException {
    if (this.jarFile == null)
      throw FILE_NOT_FOUND_EXCEPTION; 
    if (this.permission == null)
      this.permission = this.jarFile.getPermission(); 
    return this.permission;
  }
  
  public long getLastModified() {
    if (this.jarFile == null || this.jarEntryName.isEmpty())
      return 0L; 
    try {
      JarEntry entry = getJarEntry();
      return (entry != null) ? entry.getTime() : 0L;
    } catch (IOException ex) {
      return 0L;
    } 
  }
  
  static void setUseFastExceptions(boolean useFastExceptions) {
    org.springframework.boot.loader.jar.JarURLConnection.useFastExceptions.set(Boolean.valueOf(useFastExceptions));
  }
  
  static org.springframework.boot.loader.jar.JarURLConnection get(URL url, JarFile jarFile) throws IOException {
    StringSequence spec = new StringSequence(url.getFile());
    int index = indexOfRootSpec(spec, jarFile.getPathFromRoot());
    if (index == -1)
      return Boolean.TRUE.equals(useFastExceptions.get()) ? NOT_FOUND_CONNECTION : new org.springframework.boot.loader.jar.JarURLConnection(url, null, EMPTY_JAR_ENTRY_NAME); 
    int separator;
    while ((separator = spec.indexOf("!/", index)) > 0) {
      JarEntryName entryName = JarEntryName.get(spec.subSequence(index, separator));
      JarEntry jarEntry = jarFile.getJarEntry(entryName.toCharSequence());
      if (jarEntry == null)
        return notFound(jarFile, entryName); 
      jarFile = jarFile.getNestedJarFile(jarEntry);
      index = separator + "!/".length();
    } 
    JarEntryName jarEntryName = JarEntryName.get(spec, index);
    if (Boolean.TRUE.equals(useFastExceptions.get()) && !jarEntryName.isEmpty() && 
      !jarFile.containsEntry(jarEntryName.toString()))
      return NOT_FOUND_CONNECTION; 
    return new org.springframework.boot.loader.jar.JarURLConnection(url, (AbstractJarFile)jarFile.getWrapper(), jarEntryName);
  }
  
  private static int indexOfRootSpec(StringSequence file, String pathFromRoot) {
    int separatorIndex = file.indexOf("!/");
    if (separatorIndex < 0 || !file.startsWith(pathFromRoot, separatorIndex))
      return -1; 
    return separatorIndex + "!/".length() + pathFromRoot.length();
  }
  
  private static org.springframework.boot.loader.jar.JarURLConnection notFound() {
    try {
      return notFound(null, null);
    } catch (IOException ex) {
      throw new IllegalStateException(ex);
    } 
  }
  
  private static org.springframework.boot.loader.jar.JarURLConnection notFound(JarFile jarFile, JarEntryName jarEntryName) throws IOException {
    if (Boolean.TRUE.equals(useFastExceptions.get()))
      return NOT_FOUND_CONNECTION; 
    return new org.springframework.boot.loader.jar.JarURLConnection(null, (AbstractJarFile)jarFile, jarEntryName);
  }
}
